""" Constants for api.presalytics.io client """
HOST = "https://api.presalytics.io"
SITE_HOST = "https://presalytics.io"
OIDC_WELL_KNOWN = "https://auth.presalytics.io/auth/realms/presalytics/.well-known/openid-configuration"
OIDC_TOKEN = "https://auth.presalytics.io/auth/realms/presalytics/protocol/openid-connect/token"
PORT = 443
OIDC_AUTH_HOST = "https://login.presalytics.io/"
OIDC_AUTH_PORT = 443
REDIRECT_URI = "https://presalytics.io/user/login-success"
OIDC_REALM = "presalytics"
DEFAULT_CLIENT_ID = "JdypLkpW4JF7ORdNnfcul5N4mifzS5MC"
TOKEN_FILE = "token.json"
LOCALHOST_SERVER = "127.0.0.1"
LOCALHOST_PORT = 8052
LOGIN_PATH = "/accounts/login/"
API_LOGGEDIN_NEXT_URL = "/user/api-logged-in/"
STORY_VIEW_URL = "/story/view/{0}/"
STORY_MANAGE_URL = "/story/manage/{0}/"
DEFAULT_AUDIENCE = "https://api.presalytics.io/"