{% extends 'title.html' %}

{% block page_content %}
    <div class="flex-row">
        <div class="flex-column two-col">
            <div class="item-container">
                {{ widgets[widget_index.next()].to_html().decode('utf-8') }}
            </div>
        </div>
        <div class="flex-column two-col">
            <div class="item-container">
                {{ widgets[widget_index.next()].to_html().decode('utf-8') }}
            </div>
        </div>
    </div>
{% endblock %}